% Copyright is with the following author(s):
%
% (c) 2012 Juraj Oravec, Slovak University of Technology in Bratislava,
% juraj.oravec@stuba.sk
% (c) 2012 Monika Bakosova, Slovak University of Technology in Bratislava,
% monika.bakosova@stuba.sk
% ------------------------------------------------------------------------------
% Legal note:
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public
% License as published by the Free Software Foundation; either
% version 2.1 of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public
% License along with this library; if not, write to the
% Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330,
% Boston, MA 02111-1307 USA
%
% ------------------------------------------------------------------------------
%
% pid_tune(num, den, td, kp, ki, kd, range_kp, range_ki, range_kd, weight_y, weight_u)

function pid_tune(num, den, td, kp, ki, kd, range_kp, range_ki, range_kd, weight_y, weight_u)

global pid_var_kp pid_var_ki pid_var_kd pid_var_sim_num pid_var_sim_den pid_var_sim_td pid_var_weight_y pid_var_weight_u pid_var_sim_tsim

if(nargin < 7)
    range_kp = 8;
    range_ki = 8;
    range_kd = 8;
%     range_kp = kp*(1 + range_kp) - kp*(1 - range_kp);
%     range_ki = ki*(1 + range_ki) - ki*(1 - range_ki);
%     range_kd = kd*(1 + range_kd) - kd*(1 - range_kd);
    weight_y = 1;
    weight_u = 0.001;
end
    
% if(flag == 0 ) % FLAG == 0 - INIT

% Close yet opened pid_TUNE figures
%
close(findobj('Tag','pid_tune'))
close(findobj('Tag','pid_tune_fig'))

% Open new figure of pid_TUNE
%
hf1 = figure;
set(gcf,'Name','PID_TUNE',...
'Tag','pid_tune',...
'Numbertitle','off',...
'Menubar','none')
size_fig = get(hf1,'Position');
set(hf1,'Position',size_fig/2);

% Data structure generation
%
data.kp = kp;
data.ki = ki;
data.kd = kd;
data.kp_min = kp - range_kp;
data.kp_max = kp + range_kp;
data.ki_min = ki - range_ki;
data.ki_max = ki + range_ki;
data.kd_min = kd - range_kd;
data.kd_max = kd + range_kd;
dara.range_kp = range_kp;
dara.range_ki = range_ki;
dara.range_kd = range_kd;
data.num = num;
data.den = den;
data.td = td;
data.error_nom = -Inf;
data.val_c_nom = 1;
data.weight_y = weight_y;
data.weight_u = weight_u;

set(gcf,'Userdata',data);

% KP
%
t_kp = uicontrol('Style','text',...
'Tag','pid_t_kp',...
'Units','Normalized',...
'Fontweight','bold',...
'FontUnits','Points',...
'Fontsize',12,...
'Position',[0.01 0.75 0.15 0.12],...
'String','P',...
'BackgroundColor',get(gcf,'Color'));

s_kp = uicontrol('Tag','pid_s_kp',...
'Units','Normalized',...
'Position',[0.15 0.75 0.50 0.12],...
'Style','Slider',...
'Min',kp-range_kp,'Max',kp+range_kp,...
'Value',kp,...
'Sliderstep',[(abs(kp)+range_kp)/1000, (abs(kp)+range_kp)/100],...
'CallBack','pid_get_param(''s'')');

e_kp = uicontrol('Tag','pid_e_kp',...
'Units','Normalized',...
'Position',[0.70 0.75 0.25 0.12],...
'Style','edit',...
'Fontsize',12,...
'String',kp,...
'BackgroundColor','w',...
'CallBack','pid_get_param(''e'')');

% KI
%
t_ki = uicontrol('Style','text',...
'Tag','pid_t_ki',...
'Units','Normalized',...
'Fontweight','bold',...
'FontUnits','Points',...
'Fontsize',12,...
'Position',[0.01 0.55 0.15 0.12],...
'String','I',...
'BackgroundColor',get(gcf,'Color'));

s_ki = uicontrol('Tag','pid_s_ki',...
'Units','Normalized',...
'Position',[0.15 0.55 0.50 0.12],...
'Style','Slider',...
'Value',ki,...
'Min',ki-range_ki,'Max',ki+range_ki,...
'String',ki,...
'Sliderstep',[(abs(ki)+range_ki)/1000, (abs(ki)+range_ki)/100],...
'CallBack','pid_get_param(''s'')');

e_ki = uicontrol('Tag','pid_e_ki',...
'Units','Normalized',...
'Position',[0.70 0.55 0.25 0.12],...
'Style','edit',...
'Fontsize',12,...
'String',ki,...
'BackgroundColor','w',...
'CallBack','pid_get_param(''e'')');


% KD
%
t_kd = uicontrol('Style','text',...
'Tag','pid_t_kd',...
'Units','Normalized',...
'Fontweight','bold',...
'FontUnits','Points',...
'Fontsize',12,...
'Position',[0.01 0.35 0.15 0.12],...
'String','D',...
'BackgroundColor',get(gcf,'Color'));

s_kd = uicontrol('Tag','pid_s_kd',...
'Units','Normalized',...
'Position',[0.15 0.35 0.50 0.12],...
'Style','Slider',...
'Value',kd,...
'Min',kd-range_kd,'Max',kd+range_kd,...
'String',kd,...
'Sliderstep',[(abs(kd)+range_kd)/1000, (abs(kd)+range_kd)/100],...
'CallBack','pid_get_param(''s'')');

e_kd = uicontrol('Tag','pid_e_kd',...
'Units','Normalized',...
'Position',[0.70 0.35 0.25 0.12],...
'Style','edit',...
'Fontsize',12,...
'String',kd,...
'BackgroundColor','w',...
'CallBack','pid_get_param(''e'')');

% PROCES_NOM
%
t_nom = uicontrol('Style','text',...
'Tag','pid_t_nom',...
'Units','Normalized',...
'Fontweight','bold',...
'FontUnits','Points',...
'Fontsize',12,...
'Position',[0.01 0.10 0.15 0.12],...
'String','Q',...
'BackgroundColor',get(gcf,'Color'));

e_nom = uicontrol('Tag','pid_e_nom',...
'Units','Normalized',...
'Position',[0.15 0.10 0.50 0.12],...
'Style','edit',...
'Fontsize',12,...
'String',data.error_nom,...
'BackgroundColor',get(gcf,'Color'),...
'CallBack','pid_get_param(''x'')');

% Simulation of Control
%
% Simulation time TSIM evaluation
%
pid_var_sim_tsim = pid_get_simtime;
%
pid_var_sim_num = data.num;
pid_var_sim_den = data.den;
pid_var_sim_td = data.td;
pid_var_weight_y = data.weight_y;
pid_var_weight_u = data.weight_u;
%
pid_tune_sim

end % function
